"""
Generated class for EvidenceVariable. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.Annotation import *
from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.Coding import *
from fhirmodels.R5.ContactDetail import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Expression import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Quantity import *
from fhirmodels.R5.Range import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.RelatedArtifact import *
from fhirmodels.R5.Resource import *
from fhirmodels.R5.UsageContext import *


class DefinitionByTypeAndValue(FhirBaseModel):
    """Defines the characteristic using both a type and value[x] elements.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Expresses the type of characteristic
    :param CodeableConcept method: Method for how the characteristic value was determined
    :param Reference device: Device used for determining characteristic
    :param CodeableConcept valueCodeableConcept: Defines the characteristic when coupled with characteristic.type
    :param bool valueBoolean: Defines the characteristic when coupled with characteristic.type
    :param Quantity valueQuantity: Defines the characteristic when coupled with characteristic.type
    :param Range valueRange: Defines the characteristic when coupled with characteristic.type
    :param Reference valueReference: Defines the characteristic when coupled with characteristic.type
    :param str valueId: Defines the characteristic when coupled with characteristic.type
    :param CodeableConcept offset: Reference point for valueQuantity or valueRange
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "method": {"class_name": "CodeableConcept", "is_contained": False},
        "device": {"class_name": "Reference", "is_contained": False},
        "valueCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueRange": {"class_name": "Range", "is_contained": False},
        "valueReference": {"class_name": "Reference", "is_contained": False},
        "offset": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        method: list["CodeableConcept"] = None,
        device: "Reference" = None,
        valueCodeableConcept: "CodeableConcept" = None,
        valueBoolean: "bool" = None,
        valueQuantity: "Quantity" = None,
        valueRange: "Range" = None,
        valueReference: "Reference" = None,
        valueId: "str" = None,
        offset: "CodeableConcept" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.method = method or []
        self.device = device
        self.valueCodeableConcept = valueCodeableConcept
        self.valueBoolean = valueBoolean
        self.valueQuantity = valueQuantity
        self.valueRange = valueRange
        self.valueReference = valueReference
        self.valueId = valueId
        self.offset = offset

    @classmethod
    def from_dict(cls, data: dict) -> "EvidenceVariable":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "EvidenceVariable":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class DefinitionByCombination(FhirBaseModel):
    """Defines the characteristic as a combination of two or more characteristics.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: all-of | any-of | at-least | at-most | statistical | net-effect | dataset
    :param int threshold: Provides the value of "n" when "at-least" or "at-most" codes are used
    :param Characteristic characteristic: A defining factor of the characteristic
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "characteristic": {"class_name": "Characteristic", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        threshold: "int" = None,
        characteristic: list["Characteristic"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.threshold = threshold
        self.characteristic = characteristic or []

    @classmethod
    def from_dict(cls, data: dict) -> "EvidenceVariable":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "EvidenceVariable":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class TimeFromEvent(FhirBaseModel):
    """Timing in which the characteristic is determined.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str description: Human readable description
    :param Annotation note: Used for footnotes or explanatory notes
    :param CodeableConcept eventCodeableConcept: The event used as a base point (reference point) in time
    :param Reference eventReference: The event used as a base point (reference point) in time
    :param str eventDateTime: The event used as a base point (reference point) in time
    :param str eventId: The event used as a base point (reference point) in time
    :param Quantity quantity: Used to express the observation at a defined amount of time before or after the event
    :param Range range: Used to express the observation within a period before and/or after the event
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "note": {"class_name": "Annotation", "is_contained": False},
        "eventCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "eventReference": {"class_name": "Reference", "is_contained": False},
        "quantity": {"class_name": "Quantity", "is_contained": False},
        "range": {"class_name": "Range", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        description: "str" = None,
        note: list["Annotation"] = None,
        eventCodeableConcept: "CodeableConcept" = None,
        eventReference: "Reference" = None,
        eventDateTime: "str" = None,
        eventId: "str" = None,
        quantity: "Quantity" = None,
        range: "Range" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.description = description
        self.note = note or []
        self.eventCodeableConcept = eventCodeableConcept
        self.eventReference = eventReference
        self.eventDateTime = eventDateTime
        self.eventId = eventId
        self.quantity = quantity
        self.range = range

    @classmethod
    def from_dict(cls, data: dict) -> "EvidenceVariable":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "EvidenceVariable":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Characteristic(FhirBaseModel):
    """A defining factor of the EvidenceVariable. Multiple characteristics are applied with "and" semantics.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str linkId: Label for internal linking
    :param str description: Natural language description of the characteristic
    :param Annotation note: Used for footnotes or explanatory notes
    :param bool exclude: Whether the characteristic is an inclusion criterion or exclusion criterion
    :param Reference definitionReference: Defines the characteristic (without using type and value) by a Reference
    :param str definitionCanonical: Defines the characteristic (without using type and value) by a Canonical
    :param CodeableConcept definitionCodeableConcept: Defines the characteristic (without using type and value) by a CodeableConcept
    :param Expression definitionExpression: Defines the characteristic (without using type and value) by an expression
    :param str definitionId: Defines the characteristic (without using type and value) by an id
    :param DefinitionByTypeAndValue definitionByTypeAndValue: Defines the characteristic using type and value
    :param DefinitionByCombination definitionByCombination: Used to specify how two or more characteristics are combined
    :param Quantity instancesQuantity: Number of occurrences meeting the characteristic
    :param Range instancesRange: Number of occurrences meeting the characteristic
    :param Quantity durationQuantity: Length of time in which the characteristic is met
    :param Range durationRange: Length of time in which the characteristic is met
    :param TimeFromEvent timeFromEvent: Timing in which the characteristic is determined
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "note": {"class_name": "Annotation", "is_contained": False},
        "definitionReference": {"class_name": "Reference", "is_contained": False},
        "definitionCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "definitionExpression": {"class_name": "Expression", "is_contained": False},
        "definitionByTypeAndValue": {
            "class_name": "DefinitionByTypeAndValue",
            "is_contained": True,
        },
        "definitionByCombination": {
            "class_name": "DefinitionByCombination",
            "is_contained": True,
        },
        "instancesQuantity": {"class_name": "Quantity", "is_contained": False},
        "instancesRange": {"class_name": "Range", "is_contained": False},
        "durationQuantity": {"class_name": "Quantity", "is_contained": False},
        "durationRange": {"class_name": "Range", "is_contained": False},
        "timeFromEvent": {"class_name": "TimeFromEvent", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        linkId: "str" = None,
        description: "str" = None,
        note: list["Annotation"] = None,
        exclude: "bool" = None,
        definitionReference: "Reference" = None,
        definitionCanonical: "str" = None,
        definitionCodeableConcept: "CodeableConcept" = None,
        definitionExpression: "Expression" = None,
        definitionId: "str" = None,
        definitionByTypeAndValue: "DefinitionByTypeAndValue" = None,
        definitionByCombination: "DefinitionByCombination" = None,
        instancesQuantity: "Quantity" = None,
        instancesRange: "Range" = None,
        durationQuantity: "Quantity" = None,
        durationRange: "Range" = None,
        timeFromEvent: list["TimeFromEvent"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.linkId = linkId
        self.description = description
        self.note = note or []
        self.exclude = exclude
        self.definitionReference = definitionReference
        self.definitionCanonical = definitionCanonical
        self.definitionCodeableConcept = definitionCodeableConcept
        self.definitionExpression = definitionExpression
        self.definitionId = definitionId
        self.definitionByTypeAndValue = definitionByTypeAndValue
        self.definitionByCombination = definitionByCombination
        self.instancesQuantity = instancesQuantity
        self.instancesRange = instancesRange
        self.durationQuantity = durationQuantity
        self.durationRange = durationRange
        self.timeFromEvent = timeFromEvent or []

    @classmethod
    def from_dict(cls, data: dict) -> "EvidenceVariable":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "EvidenceVariable":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Category(FhirBaseModel):
    """A grouping for ordinal or polychotomous variables.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str name: Description of the grouping
    :param CodeableConcept valueCodeableConcept: Definition of the grouping
    :param Quantity valueQuantity: Definition of the grouping
    :param Range valueRange: Definition of the grouping
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "valueCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueRange": {"class_name": "Range", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        name: "str" = None,
        valueCodeableConcept: "CodeableConcept" = None,
        valueQuantity: "Quantity" = None,
        valueRange: "Range" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.name = name
        self.valueCodeableConcept = valueCodeableConcept
        self.valueQuantity = valueQuantity
        self.valueRange = valueRange

    @classmethod
    def from_dict(cls, data: dict) -> "EvidenceVariable":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "EvidenceVariable":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class EvidenceVariable(DomainResource):
    """The EvidenceVariable resource describes an element that knowledge (Evidence) is about.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param str url: Canonical identifier for this evidence variable, represented as a URI (globally unique)
    :param Identifier identifier: Additional identifier for the evidence variable
    :param str version: Business version of the evidence variable
    :param str versionAlgorithmString: How to compare versions
    :param Coding versionAlgorithmCoding: How to compare versions
    :param str name: Name for this evidence variable (computer friendly)
    :param str title: Name for this evidence variable (human friendly)
    :param str shortTitle: Title for use in informal contexts
    :param str status: draft | active | retired | unknown
    :param bool experimental: For testing purposes, not real usage
    :param str date: Date last changed
    :param str publisher: Name of the publisher/steward (organization or individual)
    :param ContactDetail contact: Contact details for the publisher
    :param str description: Natural language description of the evidence variable
    :param Annotation note: Used for footnotes or explanatory notes
    :param UsageContext useContext: The context that the content is intended to support
    :param str purpose: Why this EvidenceVariable is defined
    :param str copyright: Use and/or publishing restrictions
    :param str copyrightLabel: Copyright holder and year(s)
    :param str approvalDate: When the resource was approved by publisher
    :param str lastReviewDate: When the resource was last reviewed by the publisher
    :param Period effectivePeriod: When the resource is expected to be used
    :param ContactDetail author: Who authored the content
    :param ContactDetail editor: Who edited the content
    :param ContactDetail reviewer: Who reviewed the content
    :param ContactDetail endorser: Who endorsed the content
    :param RelatedArtifact relatedArtifact: Additional documentation, citations, etc
    :param bool actual: Actual or conceptual
    :param Characteristic characteristic: A defining factor of the EvidenceVariable
    :param str handling: continuous | dichotomous | ordinal | polychotomous
    :param Category category: A grouping for ordinal or polychotomous variables
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "versionAlgorithmCoding": {"class_name": "Coding", "is_contained": False},
        "contact": {"class_name": "ContactDetail", "is_contained": False},
        "note": {"class_name": "Annotation", "is_contained": False},
        "useContext": {"class_name": "UsageContext", "is_contained": False},
        "effectivePeriod": {"class_name": "Period", "is_contained": False},
        "author": {"class_name": "ContactDetail", "is_contained": False},
        "editor": {"class_name": "ContactDetail", "is_contained": False},
        "reviewer": {"class_name": "ContactDetail", "is_contained": False},
        "endorser": {"class_name": "ContactDetail", "is_contained": False},
        "relatedArtifact": {"class_name": "RelatedArtifact", "is_contained": False},
        "characteristic": {"class_name": "Characteristic", "is_contained": True},
        "category": {"class_name": "Category", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        url: "str" = None,
        identifier: list["Identifier"] = None,
        version: "str" = None,
        versionAlgorithmString: "str" = None,
        versionAlgorithmCoding: "Coding" = None,
        name: "str" = None,
        title: "str" = None,
        shortTitle: "str" = None,
        status: "str" = None,
        experimental: "bool" = None,
        date: "str" = None,
        publisher: "str" = None,
        contact: list["ContactDetail"] = None,
        description: "str" = None,
        note: list["Annotation"] = None,
        useContext: list["UsageContext"] = None,
        purpose: "str" = None,
        copyright: "str" = None,
        copyrightLabel: "str" = None,
        approvalDate: "str" = None,
        lastReviewDate: "str" = None,
        effectivePeriod: "Period" = None,
        author: list["ContactDetail"] = None,
        editor: list["ContactDetail"] = None,
        reviewer: list["ContactDetail"] = None,
        endorser: list["ContactDetail"] = None,
        relatedArtifact: list["RelatedArtifact"] = None,
        actual: "bool" = None,
        characteristic: list["Characteristic"] = None,
        handling: "str" = None,
        category: list["Category"] = None,
    ):

        self.resourceType = "EvidenceVariable"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.url = url
        self.identifier = identifier or []
        self.version = version
        self.versionAlgorithmString = versionAlgorithmString
        self.versionAlgorithmCoding = versionAlgorithmCoding
        self.name = name
        self.title = title
        self.shortTitle = shortTitle
        self.status = status
        self.experimental = experimental
        self.date = date
        self.publisher = publisher
        self.contact = contact or []
        self.description = description
        self.note = note or []
        self.useContext = useContext or []
        self.purpose = purpose
        self.copyright = copyright
        self.copyrightLabel = copyrightLabel
        self.approvalDate = approvalDate
        self.lastReviewDate = lastReviewDate
        self.effectivePeriod = effectivePeriod
        self.author = author or []
        self.editor = editor or []
        self.reviewer = reviewer or []
        self.endorser = endorser or []
        self.relatedArtifact = relatedArtifact or []
        self.actual = actual
        self.characteristic = characteristic or []
        self.handling = handling
        self.category = category or []

    @classmethod
    def from_dict(cls, data: dict) -> "EvidenceVariable":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "EvidenceVariable":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for ProductShelfLife. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.generator import FhirBaseModel
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.Duration import *
from fhirmodels.R5.Extension import *


class ProductShelfLife(FhirBaseModel):
    """ProductShelfLife Type: The shelf-life and storage information for a medicinal product item or container can be described using this class.
    :param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: This describes the shelf life, taking into account various scenarios such as shelf life of the packaged Medicinal Product itself, shelf life after transformation where necessary and shelf life after the first opening of a bottle, etc. The shelf life type shall be specified using an appropriate controlled vocabulary The controlled term and the controlled term identifier shall be specified
    :param Duration periodDuration: The shelf life time period can be specified using a numerical value for the period of time and its unit of time measurement The unit of measurement shall be specified in accordance with ISO 11240 and the resulting terminology The symbol and the symbol identifier shall be used
    :param str periodString: The shelf life time period can be specified using a numerical value for the period of time and its unit of time measurement The unit of measurement shall be specified in accordance with ISO 11240 and the resulting terminology The symbol and the symbol identifier shall be used
    :param CodeableConcept specialPrecautionsForStorage: Special precautions for storage, if any, can be specified using an appropriate controlled vocabulary The controlled term and the controlled term identifier shall be specified
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "periodDuration": {"class_name": "Duration", "is_contained": False},
        "specialPrecautionsForStorage": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        periodDuration: "Duration" = None,
        periodString: "str" = None,
        specialPrecautionsForStorage: list["CodeableConcept"] = None,
    ):

        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.periodDuration = periodDuration
        self.periodString = periodString
        self.specialPrecautionsForStorage = specialPrecautionsForStorage or []

    @classmethod
    def from_dict(cls, data: dict) -> "ProductShelfLife":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ProductShelfLife":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Appointment. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.Annotation import *
from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.CodeableReference import *
from fhirmodels.R5.Coding import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *
from fhirmodels.R5.VirtualServiceDetail import *


class Participant(FhirBaseModel):
    """List of participants involved in the appointment.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Role of participant in the appointment
    :param Period period: Participation period of the actor
    :param Reference actor: The individual, device, location, or service participating in the appointment
    :param bool required: The participant is required to attend (optional when false)
    :param str status: accepted | declined | tentative | needs-action
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
        "actor": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: list["CodeableConcept"] = None,
        period: "Period" = None,
        actor: "Reference" = None,
        required: "bool" = None,
        status: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type or []
        self.period = period
        self.actor = actor
        self.required = required
        self.status = status

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class WeeklyTemplate(FhirBaseModel):
    """Information about weekly recurring appointments.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param bool monday: Recurs on Mondays
    :param bool tuesday: Recurs on Tuesday
    :param bool wednesday: Recurs on Wednesday
    :param bool thursday: Recurs on Thursday
    :param bool friday: Recurs on Friday
    :param bool saturday: Recurs on Saturday
    :param bool sunday: Recurs on Sunday
    :param int weekInterval: Recurs every nth week
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        monday: "bool" = None,
        tuesday: "bool" = None,
        wednesday: "bool" = None,
        thursday: "bool" = None,
        friday: "bool" = None,
        saturday: "bool" = None,
        sunday: "bool" = None,
        weekInterval: "int" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.monday = monday
        self.tuesday = tuesday
        self.wednesday = wednesday
        self.thursday = thursday
        self.friday = friday
        self.saturday = saturday
        self.sunday = sunday
        self.weekInterval = weekInterval

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class MonthlyTemplate(FhirBaseModel):
    """Information about monthly recurring appointments.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param int dayOfMonth: Recurs on a specific day of the month
    :param Coding nthWeekOfMonth: Indicates which week of the month the appointment should occur
    :param Coding dayOfWeek: Indicates which day of the week the appointment should occur
    :param int monthInterval: Recurs every nth month
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "nthWeekOfMonth": {"class_name": "Coding", "is_contained": False},
        "dayOfWeek": {"class_name": "Coding", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        dayOfMonth: "int" = None,
        nthWeekOfMonth: "Coding" = None,
        dayOfWeek: "Coding" = None,
        monthInterval: "int" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.dayOfMonth = dayOfMonth
        self.nthWeekOfMonth = nthWeekOfMonth
        self.dayOfWeek = dayOfWeek
        self.monthInterval = monthInterval

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class YearlyTemplate(FhirBaseModel):
    """Information about yearly recurring appointments.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param int yearInterval: Recurs every nth year
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        yearInterval: "int" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.yearInterval = yearInterval

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class RecurrenceTemplate(FhirBaseModel):
    """The details of the recurrence pattern or template that is used to generate recurring appointments.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept timezone: The timezone of the occurrences
    :param CodeableConcept recurrenceType: The frequency of the recurrence
    :param str lastOccurrenceDate: The date when the recurrence should end
    :param int occurrenceCount: The number of planned occurrences
    :param str occurrenceDate: Specific dates for a recurring set of appointments (no template)
    :param WeeklyTemplate weeklyTemplate: Information about weekly recurring appointments
    :param MonthlyTemplate monthlyTemplate: Information about monthly recurring appointments
    :param YearlyTemplate yearlyTemplate: Information about yearly recurring appointments
    :param str excludingDate: Any dates that should be excluded from the series
    :param int excludingRecurrenceId: Any recurrence IDs that should be excluded from the recurrence
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "timezone": {"class_name": "CodeableConcept", "is_contained": False},
        "recurrenceType": {"class_name": "CodeableConcept", "is_contained": False},
        "weeklyTemplate": {"class_name": "WeeklyTemplate", "is_contained": True},
        "monthlyTemplate": {"class_name": "MonthlyTemplate", "is_contained": True},
        "yearlyTemplate": {"class_name": "YearlyTemplate", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        timezone: "CodeableConcept" = None,
        recurrenceType: "CodeableConcept" = None,
        lastOccurrenceDate: "str" = None,
        occurrenceCount: "int" = None,
        occurrenceDate: list["str"] = None,
        weeklyTemplate: "WeeklyTemplate" = None,
        monthlyTemplate: "MonthlyTemplate" = None,
        yearlyTemplate: "YearlyTemplate" = None,
        excludingDate: list["str"] = None,
        excludingRecurrenceId: list["int"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.timezone = timezone
        self.recurrenceType = recurrenceType
        self.lastOccurrenceDate = lastOccurrenceDate
        self.occurrenceCount = occurrenceCount
        self.occurrenceDate = occurrenceDate or []
        self.weeklyTemplate = weeklyTemplate
        self.monthlyTemplate = monthlyTemplate
        self.yearlyTemplate = yearlyTemplate
        self.excludingDate = excludingDate or []
        self.excludingRecurrenceId = excludingRecurrenceId or []

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Appointment(DomainResource):
    """A booking of a healthcare event among patient(s), practitioner(s), related person(s) and/or device(s) for a specific date/time. This may result in one or more Encounter(s).
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: External Ids for this item
    :param str status: proposed | pending | booked | arrived | fulfilled | cancelled | noshow | entered-in-error | checked-in | waitlist
    :param CodeableConcept cancellationReason: The coded reason for the appointment being cancelled
    :param CodeableConcept _class: Classification when becoming an encounter
    :param CodeableConcept serviceCategory: A broad categorization of the service that is to be performed during this appointment
    :param CodeableReference serviceType: The specific service that is to be performed during this appointment
    :param CodeableConcept specialty: The specialty of a practitioner that would be required to perform the service requested in this appointment
    :param CodeableConcept appointmentType: The style of appointment or patient that has been booked in the slot (not service type)
    :param CodeableReference reason: Reason this appointment is scheduled
    :param CodeableConcept priority: Used to make informed decisions if needing to re-prioritize
    :param str description: Shown on a subject line in a meeting request, or appointment list
    :param Reference replaces: Appointment replaced by this Appointment
    :param VirtualServiceDetail virtualService: Connection details of a virtual service (e.g. conference call)
    :param Reference supportingInformation: Additional information to support the appointment
    :param Reference previousAppointment: The previous appointment in a series
    :param Reference originatingAppointment: The originating appointment in a recurring set of appointments
    :param str start: When appointment is to take place
    :param str end: When appointment is to conclude
    :param int minutesDuration: Can be less than start/end (e.g. estimate)
    :param Period requestedPeriod: Potential date/time interval(s) requested to allocate the appointment within
    :param Reference slot: The slots that this appointment is filling
    :param Reference account: The set of accounts that may be used for billing for this Appointment
    :param str created: The date that this appointment was initially created
    :param str cancellationDate: When the appointment was cancelled
    :param Annotation note: Additional comments
    :param CodeableReference patientInstruction: Detailed information and instructions for the patient
    :param Reference basedOn: The request this appointment is allocated to assess
    :param Reference subject: The patient or group associated with the appointment
    :param Participant participant: Participants involved in appointment
    :param int recurrenceId: The sequence number in the recurrence
    :param bool occurrenceChanged: Indicates that this appointment varies from a recurrence pattern
    :param RecurrenceTemplate recurrenceTemplate: Details of the recurrence pattern/template used to generate occurrences
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "cancellationReason": {"class_name": "CodeableConcept", "is_contained": False},
        "_class": {"class_name": "CodeableConcept", "is_contained": False},
        "serviceCategory": {"class_name": "CodeableConcept", "is_contained": False},
        "serviceType": {"class_name": "CodeableReference", "is_contained": False},
        "specialty": {"class_name": "CodeableConcept", "is_contained": False},
        "appointmentType": {"class_name": "CodeableConcept", "is_contained": False},
        "reason": {"class_name": "CodeableReference", "is_contained": False},
        "priority": {"class_name": "CodeableConcept", "is_contained": False},
        "replaces": {"class_name": "Reference", "is_contained": False},
        "virtualService": {"class_name": "VirtualServiceDetail", "is_contained": False},
        "supportingInformation": {"class_name": "Reference", "is_contained": False},
        "previousAppointment": {"class_name": "Reference", "is_contained": False},
        "originatingAppointment": {"class_name": "Reference", "is_contained": False},
        "requestedPeriod": {"class_name": "Period", "is_contained": False},
        "slot": {"class_name": "Reference", "is_contained": False},
        "account": {"class_name": "Reference", "is_contained": False},
        "note": {"class_name": "Annotation", "is_contained": False},
        "patientInstruction": {
            "class_name": "CodeableReference",
            "is_contained": False,
        },
        "basedOn": {"class_name": "Reference", "is_contained": False},
        "subject": {"class_name": "Reference", "is_contained": False},
        "participant": {"class_name": "Participant", "is_contained": True},
        "recurrenceTemplate": {
            "class_name": "RecurrenceTemplate",
            "is_contained": True,
        },
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        cancellationReason: "CodeableConcept" = None,
        _class: list["CodeableConcept"] = None,
        serviceCategory: list["CodeableConcept"] = None,
        serviceType: list["CodeableReference"] = None,
        specialty: list["CodeableConcept"] = None,
        appointmentType: "CodeableConcept" = None,
        reason: list["CodeableReference"] = None,
        priority: "CodeableConcept" = None,
        description: "str" = None,
        replaces: list["Reference"] = None,
        virtualService: list["VirtualServiceDetail"] = None,
        supportingInformation: list["Reference"] = None,
        previousAppointment: "Reference" = None,
        originatingAppointment: "Reference" = None,
        start: "str" = None,
        end: "str" = None,
        minutesDuration: "int" = None,
        requestedPeriod: list["Period"] = None,
        slot: list["Reference"] = None,
        account: list["Reference"] = None,
        created: "str" = None,
        cancellationDate: "str" = None,
        note: list["Annotation"] = None,
        patientInstruction: list["CodeableReference"] = None,
        basedOn: list["Reference"] = None,
        subject: "Reference" = None,
        participant: list["Participant"] = None,
        recurrenceId: "int" = None,
        occurrenceChanged: "bool" = None,
        recurrenceTemplate: list["RecurrenceTemplate"] = None,
    ):

        self.resourceType = "Appointment"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.cancellationReason = cancellationReason
        self._class = _class or []
        self.serviceCategory = serviceCategory or []
        self.serviceType = serviceType or []
        self.specialty = specialty or []
        self.appointmentType = appointmentType
        self.reason = reason or []
        self.priority = priority
        self.description = description
        self.replaces = replaces or []
        self.virtualService = virtualService or []
        self.supportingInformation = supportingInformation or []
        self.previousAppointment = previousAppointment
        self.originatingAppointment = originatingAppointment
        self.start = start
        self.end = end
        self.minutesDuration = minutesDuration
        self.requestedPeriod = requestedPeriod or []
        self.slot = slot or []
        self.account = account or []
        self.created = created
        self.cancellationDate = cancellationDate
        self.note = note or []
        self.patientInstruction = patientInstruction or []
        self.basedOn = basedOn or []
        self.subject = subject
        self.participant = participant or []
        self.recurrenceId = recurrenceId
        self.occurrenceChanged = occurrenceChanged
        self.recurrenceTemplate = recurrenceTemplate or []

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Provenance. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.CodeableReference import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *
from fhirmodels.R5.Signature import *


class Agent(FhirBaseModel):
    """An actor taking a role in an activity  for which it can be assigned some degree of responsibility for the activity taking place.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: How the agent participated
    :param CodeableConcept role: What the agents role was
    :param Reference who: The agent that participated in the event
    :param Reference onBehalfOf: The agent that delegated
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "role": {"class_name": "CodeableConcept", "is_contained": False},
        "who": {"class_name": "Reference", "is_contained": False},
        "onBehalfOf": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        role: list["CodeableConcept"] = None,
        who: "Reference" = None,
        onBehalfOf: "Reference" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.role = role or []
        self.who = who
        self.onBehalfOf = onBehalfOf

    @classmethod
    def from_dict(cls, data: dict) -> "Provenance":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Provenance":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Entity(FhirBaseModel):
    """An entity used in this activity.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str role: revision | quotation | source | instantiates | removal
    :param Reference what: Identity of entity
    :param Agent agent: Entity is attributed to this agent
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "what": {"class_name": "Reference", "is_contained": False},
        "agent": {"class_name": "Agent", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        role: "str" = None,
        what: "Reference" = None,
        agent: list["Agent"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.role = role
        self.what = what
        self.agent = agent or []

    @classmethod
    def from_dict(cls, data: dict) -> "Provenance":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Provenance":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Provenance(DomainResource):
    """Provenance of a resource is a record that describes entities and processes involved in producing and delivering or otherwise influencing that resource. Provenance provides a critical foundation for assessing authenticity, enabling trust, and allowing reproducibility. Provenance assertions are a form of contextual metadata and can themselves become important records with their own provenance. Provenance statement indicates clinical significance in terms of confidence in authenticity, reliability, and trustworthiness, integrity, and stage in lifecycle (e.g. Document Completion - has the artifact been legally authenticated), all of which may impact security, privacy, and trust policies.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Reference target: Target Reference(s) (usually version specific)
    :param Period occurredPeriod: When the activity occurred
    :param str occurredDateTime: When the activity occurred
    :param str recorded: When the activity was recorded / updated
    :param str policy: Policy or plan the activity was defined by
    :param Reference location: Where the activity occurred, if relevant
    :param CodeableReference authorization: Authorization (purposeOfUse) related to the event
    :param CodeableConcept activity: Activity that occurred
    :param Reference basedOn: Workflow authorization within which this event occurred
    :param Reference patient: The patient is the subject of the data created/updated (.target) by the activity
    :param Reference encounter: Encounter within which this event occurred or which the event is tightly associated
    :param Agent agent: Actor involved
    :param Entity entity: An entity used in this activity
    :param Signature signature: Signature on target
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "target": {"class_name": "Reference", "is_contained": False},
        "occurredPeriod": {"class_name": "Period", "is_contained": False},
        "location": {"class_name": "Reference", "is_contained": False},
        "authorization": {"class_name": "CodeableReference", "is_contained": False},
        "activity": {"class_name": "CodeableConcept", "is_contained": False},
        "basedOn": {"class_name": "Reference", "is_contained": False},
        "patient": {"class_name": "Reference", "is_contained": False},
        "encounter": {"class_name": "Reference", "is_contained": False},
        "agent": {"class_name": "Agent", "is_contained": True},
        "entity": {"class_name": "Entity", "is_contained": True},
        "signature": {"class_name": "Signature", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        target: list["Reference"] = None,
        occurredPeriod: "Period" = None,
        occurredDateTime: "str" = None,
        recorded: "str" = None,
        policy: list["str"] = None,
        location: "Reference" = None,
        authorization: list["CodeableReference"] = None,
        activity: "CodeableConcept" = None,
        basedOn: list["Reference"] = None,
        patient: "Reference" = None,
        encounter: "Reference" = None,
        agent: list["Agent"] = None,
        entity: list["Entity"] = None,
        signature: list["Signature"] = None,
    ):

        self.resourceType = "Provenance"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.target = target or []
        self.occurredPeriod = occurredPeriod
        self.occurredDateTime = occurredDateTime
        self.recorded = recorded
        self.policy = policy or []
        self.location = location
        self.authorization = authorization or []
        self.activity = activity
        self.basedOn = basedOn or []
        self.patient = patient
        self.encounter = encounter
        self.agent = agent or []
        self.entity = entity or []
        self.signature = signature or []

    @classmethod
    def from_dict(cls, data: dict) -> "Provenance":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Provenance":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for DeviceDefinition. 
Time: 2024-06-14 18:37:17
"""

from fhirmodels.R4.Annotation import *
from fhirmodels.R4.BackboneElement import *
from fhirmodels.R4.CodeableConcept import *
from fhirmodels.R4.ContactPoint import *
from fhirmodels.R4.DomainResource import *
from fhirmodels.R4.Extension import *
from fhirmodels.R4.Identifier import *
from fhirmodels.R4.Meta import *
from fhirmodels.R4.Narrative import *
from fhirmodels.R4.ProdCharacteristic import *
from fhirmodels.R4.ProductShelfLife import *
from fhirmodels.R4.Quantity import *
from fhirmodels.R4.Reference import *
from fhirmodels.R4.Resource import *


class UdiDeviceIdentifier(FhirBaseModel):
    """Unique device identifier (UDI) assigned to device label or package.  Note that the Device may include multiple udiCarriers as it either may include just the udiCarrier for the jurisdiction it is sold, or for multiple jurisdictions it could have been sold.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str deviceIdentifier: The identifier that is to be associated with every Device that references this DeviceDefintiion for the issuer and jurisdication porvided in the DeviceDefinition.udiDeviceIdentifier
    :param str issuer: The organization that assigns the identifier algorithm
    :param str jurisdiction: The jurisdiction to which the deviceIdentifier applies
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        deviceIdentifier: "str" = None,
        issuer: "str" = None,
        jurisdiction: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.deviceIdentifier = deviceIdentifier
        self.issuer = issuer
        self.jurisdiction = jurisdiction

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class DeviceName(FhirBaseModel):
    """A name given to the device to identify it.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str name: The name of the device
    :param str type: udi-label-name | user-friendly-name | patient-reported-name | manufacturer-name | model-name | other
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        name: "str" = None,
        type: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.name = name
        self.type = type

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Specialization(FhirBaseModel):
    """The capabilities supported on a  device, the standards to which the device conforms for a particular purpose, and used for the communication.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str systemType: The standard that is used to operate and communicate
    :param str version: The version of the standard that is used to operate and communicate
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        systemType: "str" = None,
        version: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.systemType = systemType
        self.version = version

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Capability(FhirBaseModel):
    """Device capabilities.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Type of capability
    :param CodeableConcept description: Description of capability
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "description": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        description: list["CodeableConcept"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.description = description or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Property(FhirBaseModel):
    """The actual configuration settings of a device as it actually operates, e.g., regulation status, time properties.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Code that specifies the property DeviceDefinitionPropetyCode (Extensible)
    :param Quantity valueQuantity: Property value as a quantity
    :param CodeableConcept valueCode: Property value as a code, e.g., NTP4 (synced to NTP)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueCode": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        valueQuantity: list["Quantity"] = None,
        valueCode: list["CodeableConcept"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.valueQuantity = valueQuantity or []
        self.valueCode = valueCode or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Material(FhirBaseModel):
    """A substance used to create the material(s) of which the device is made.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept substance: The substance
    :param bool alternate: Indicates an alternative material of the device
    :param bool allergenicIndicator: Whether the substance is a known or suspected allergen
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "substance": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        substance: "CodeableConcept" = None,
        alternate: "bool" = None,
        allergenicIndicator: "bool" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.substance = substance
        self.alternate = alternate
        self.allergenicIndicator = allergenicIndicator

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class DeviceDefinition(DomainResource):
    """The characteristics, operational status and capabilities of a medical-related component of a medical device.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Instance identifier
    :param UdiDeviceIdentifier udiDeviceIdentifier: Unique Device Identifier (UDI) Barcode string
    :param str manufacturerString: Name of device manufacturer
    :param Reference manufacturerReference: Name of device manufacturer
    :param DeviceName deviceName: A name given to the device to identify it
    :param str modelNumber: The model number for the device
    :param CodeableConcept type: What kind of device or device system this is
    :param Specialization specialization: The capabilities supported on a  device, the standards to which the device conforms for a particular purpose, and used for the communication
    :param str version: Available versions
    :param CodeableConcept safety: Safety characteristics of the device
    :param ProductShelfLife shelfLifeStorage: Shelf Life and storage information
    :param ProdCharacteristic physicalCharacteristics: Dimensions, color etc.
    :param CodeableConcept languageCode: Language code for the human-readable text strings produced by the device (all supported)
    :param Capability capability: Device capabilities
    :param Property property: The actual configuration settings of a device as it actually operates, e.g., regulation status, time properties
    :param Reference owner: Organization responsible for device
    :param ContactPoint contact: Details for human/organization for support
    :param str url: Network address to contact device
    :param str onlineInformation: Access to on-line information
    :param Annotation note: Device notes and comments
    :param Quantity quantity: The quantity of the device present in the packaging (e.g. the number of devices present in a pack, or the number of devices in the same package of the medicinal product)
    :param Reference parentDevice: The parent device it can be part of
    :param Material material: A substance used to create the material(s) of which the device is made
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "udiDeviceIdentifier": {
            "class_name": "UdiDeviceIdentifier",
            "is_contained": True,
        },
        "manufacturerReference": {"class_name": "Reference", "is_contained": False},
        "deviceName": {"class_name": "DeviceName", "is_contained": True},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "specialization": {"class_name": "Specialization", "is_contained": True},
        "safety": {"class_name": "CodeableConcept", "is_contained": False},
        "shelfLifeStorage": {"class_name": "ProductShelfLife", "is_contained": False},
        "physicalCharacteristics": {
            "class_name": "ProdCharacteristic",
            "is_contained": False,
        },
        "languageCode": {"class_name": "CodeableConcept", "is_contained": False},
        "capability": {"class_name": "Capability", "is_contained": True},
        "property": {"class_name": "Property", "is_contained": True},
        "owner": {"class_name": "Reference", "is_contained": False},
        "contact": {"class_name": "ContactPoint", "is_contained": False},
        "note": {"class_name": "Annotation", "is_contained": False},
        "quantity": {"class_name": "Quantity", "is_contained": False},
        "parentDevice": {"class_name": "Reference", "is_contained": False},
        "material": {"class_name": "Material", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        udiDeviceIdentifier: list["UdiDeviceIdentifier"] = None,
        manufacturerString: "str" = None,
        manufacturerReference: "Reference" = None,
        deviceName: list["DeviceName"] = None,
        modelNumber: "str" = None,
        type: "CodeableConcept" = None,
        specialization: list["Specialization"] = None,
        version: list["str"] = None,
        safety: list["CodeableConcept"] = None,
        shelfLifeStorage: list["ProductShelfLife"] = None,
        physicalCharacteristics: "ProdCharacteristic" = None,
        languageCode: list["CodeableConcept"] = None,
        capability: list["Capability"] = None,
        property: list["Property"] = None,
        owner: "Reference" = None,
        contact: list["ContactPoint"] = None,
        url: "str" = None,
        onlineInformation: "str" = None,
        note: list["Annotation"] = None,
        quantity: "Quantity" = None,
        parentDevice: "Reference" = None,
        material: list["Material"] = None,
    ):

        self.resourceType = "DeviceDefinition"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.udiDeviceIdentifier = udiDeviceIdentifier or []
        self.manufacturerString = manufacturerString
        self.manufacturerReference = manufacturerReference
        self.deviceName = deviceName or []
        self.modelNumber = modelNumber
        self.type = type
        self.specialization = specialization or []
        self.version = version or []
        self.safety = safety or []
        self.shelfLifeStorage = shelfLifeStorage or []
        self.physicalCharacteristics = physicalCharacteristics
        self.languageCode = languageCode or []
        self.capability = capability or []
        self.property = property or []
        self.owner = owner
        self.contact = contact or []
        self.url = url
        self.onlineInformation = onlineInformation
        self.note = note or []
        self.quantity = quantity
        self.parentDevice = parentDevice
        self.material = material or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for QuestionnaireResponse. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.Attachment import *
from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.Coding import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Quantity import *
from fhirmodels.R4B.Reference import *
from fhirmodels.R4B.Resource import *


class Answer(FhirBaseModel):
    """The respondent's answer(s) to the question.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param bool valueBoolean: Single-valued answer to the question
    :param float valueDecimal: Single-valued answer to the question
    :param int valueInteger: Single-valued answer to the question
    :param str valueDate: Single-valued answer to the question
    :param str valueDateTime: Single-valued answer to the question
    :param str valueTime: Single-valued answer to the question
    :param str valueString: Single-valued answer to the question
    :param str valueUri: Single-valued answer to the question
    :param Attachment valueAttachment: Single-valued answer to the question
    :param Coding valueCoding: Single-valued answer to the question
    :param Quantity valueQuantity: Single-valued answer to the question
    :param Reference valueReference: Single-valued answer to the question
    :param Item item: Nested groups and questions
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "valueAttachment": {"class_name": "Attachment", "is_contained": False},
        "valueCoding": {"class_name": "Coding", "is_contained": False},
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueReference": {"class_name": "Reference", "is_contained": False},
        "item": {"class_name": "Item", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        valueBoolean: "bool" = None,
        valueDecimal: "float" = None,
        valueInteger: "int" = None,
        valueDate: "str" = None,
        valueDateTime: "str" = None,
        valueTime: "str" = None,
        valueString: "str" = None,
        valueUri: "str" = None,
        valueAttachment: "Attachment" = None,
        valueCoding: "Coding" = None,
        valueQuantity: "Quantity" = None,
        valueReference: "Reference" = None,
        item: list["Item"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.valueBoolean = valueBoolean
        self.valueDecimal = valueDecimal
        self.valueInteger = valueInteger
        self.valueDate = valueDate
        self.valueDateTime = valueDateTime
        self.valueTime = valueTime
        self.valueString = valueString
        self.valueUri = valueUri
        self.valueAttachment = valueAttachment
        self.valueCoding = valueCoding
        self.valueQuantity = valueQuantity
        self.valueReference = valueReference
        self.item = item or []

    @classmethod
    def from_dict(cls, data: dict) -> "QuestionnaireResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "QuestionnaireResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Item(FhirBaseModel):
    """A group or question item from the original questionnaire for which answers are provided.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str linkId: Pointer to specific item from Questionnaire
    :param str definition: ElementDefinition - details for the item
    :param str text: Name for group or question text
    :param Answer answer: The response(s) to the question
    :param Item item: Nested questionnaire response items
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "answer": {"class_name": "Answer", "is_contained": True},
        "item": {"class_name": "Item", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        linkId: "str" = None,
        definition: "str" = None,
        text: "str" = None,
        answer: list["Answer"] = None,
        item: list["Item"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.linkId = linkId
        self.definition = definition
        self.text = text
        self.answer = answer or []
        self.item = item or []

    @classmethod
    def from_dict(cls, data: dict) -> "QuestionnaireResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "QuestionnaireResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class QuestionnaireResponse(DomainResource):
    """A structured set of questions and their answers. The questions are ordered and grouped into coherent subsets, corresponding to the structure of the grouping of the questionnaire being responded to.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Unique id for this set of answers
    :param Reference basedOn: Request fulfilled by this QuestionnaireResponse
    :param Reference partOf: Part of this action
    :param str questionnaire: Form being answered
    :param str status: in-progress | completed | amended | entered-in-error | stopped
    :param Reference subject: The subject of the questions
    :param Reference encounter: Encounter created as part of
    :param str authored: Date the answers were gathered
    :param Reference author: Person who received and recorded the answers
    :param Reference source: The person who answered the questions
    :param Item item: Groups and questions
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "basedOn": {"class_name": "Reference", "is_contained": False},
        "partOf": {"class_name": "Reference", "is_contained": False},
        "subject": {"class_name": "Reference", "is_contained": False},
        "encounter": {"class_name": "Reference", "is_contained": False},
        "author": {"class_name": "Reference", "is_contained": False},
        "source": {"class_name": "Reference", "is_contained": False},
        "item": {"class_name": "Item", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: "Identifier" = None,
        basedOn: list["Reference"] = None,
        partOf: list["Reference"] = None,
        questionnaire: "str" = None,
        status: "str" = None,
        subject: "Reference" = None,
        encounter: "Reference" = None,
        authored: "str" = None,
        author: "Reference" = None,
        source: "Reference" = None,
        item: list["Item"] = None,
    ):

        self.resourceType = "QuestionnaireResponse"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier
        self.basedOn = basedOn or []
        self.partOf = partOf or []
        self.questionnaire = questionnaire
        self.status = status
        self.subject = subject
        self.encounter = encounter
        self.authored = authored
        self.author = author
        self.source = source
        self.item = item or []

    @classmethod
    def from_dict(cls, data: dict) -> "QuestionnaireResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "QuestionnaireResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for MedicinalProductPharmaceutical. 
Time: 2024-06-14 18:37:17
"""

from fhirmodels.R4.BackboneElement import *
from fhirmodels.R4.CodeableConcept import *
from fhirmodels.R4.DomainResource import *
from fhirmodels.R4.Duration import *
from fhirmodels.R4.Extension import *
from fhirmodels.R4.Identifier import *
from fhirmodels.R4.Meta import *
from fhirmodels.R4.Narrative import *
from fhirmodels.R4.Quantity import *
from fhirmodels.R4.Ratio import *
from fhirmodels.R4.Reference import *
from fhirmodels.R4.Resource import *


class Characteristics(FhirBaseModel):
    """Characteristics e.g. a products onset of action.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept code: A coded characteristic
    :param CodeableConcept status: The status of characteristic e.g. assigned or pending
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "status": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "CodeableConcept" = None,
        status: "CodeableConcept" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.status = status

    @classmethod
    def from_dict(cls, data: dict) -> "MedicinalProductPharmaceutical":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "MedicinalProductPharmaceutical":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class WithdrawalPeriod(FhirBaseModel):
    """A species specific time during which consumption of animal product is not appropriate.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept tissue: Coded expression for the type of tissue for which the withdrawal period applues, e.g. meat, milk
    :param Quantity value: A value for the time
    :param str supportingInformation: Extra information about the withdrawal period
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "tissue": {"class_name": "CodeableConcept", "is_contained": False},
        "value": {"class_name": "Quantity", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        tissue: "CodeableConcept" = None,
        value: "Quantity" = None,
        supportingInformation: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.tissue = tissue
        self.value = value
        self.supportingInformation = supportingInformation

    @classmethod
    def from_dict(cls, data: dict) -> "MedicinalProductPharmaceutical":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "MedicinalProductPharmaceutical":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class TargetSpecies(FhirBaseModel):
    """A species for which this route applies.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept code: Coded expression for the species
    :param WithdrawalPeriod withdrawalPeriod: A species specific time during which consumption of animal product is not appropriate
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "withdrawalPeriod": {"class_name": "WithdrawalPeriod", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "CodeableConcept" = None,
        withdrawalPeriod: list["WithdrawalPeriod"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.withdrawalPeriod = withdrawalPeriod or []

    @classmethod
    def from_dict(cls, data: dict) -> "MedicinalProductPharmaceutical":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "MedicinalProductPharmaceutical":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class RouteOfAdministration(FhirBaseModel):
    """The path by which the pharmaceutical product is taken into or makes contact with the body.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept code: Coded expression for the route
    :param Quantity firstDose: The first dose (dose quantity) administered in humans can be specified, for a product under investigation, using a numerical value and its unit of measurement
    :param Quantity maxSingleDose: The maximum single dose that can be administered as per the protocol of a clinical trial can be specified using a numerical value and its unit of measurement
    :param Quantity maxDosePerDay: The maximum dose per day (maximum dose quantity to be administered in any one 24-h period) that can be administered as per the protocol referenced in the clinical trial authorisation
    :param Ratio maxDosePerTreatmentPeriod: The maximum dose per treatment period that can be administered as per the protocol referenced in the clinical trial authorisation
    :param Duration maxTreatmentPeriod: The maximum treatment period during which an Investigational Medicinal Product can be administered as per the protocol referenced in the clinical trial authorisation
    :param TargetSpecies targetSpecies: A species for which this route applies
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "firstDose": {"class_name": "Quantity", "is_contained": False},
        "maxSingleDose": {"class_name": "Quantity", "is_contained": False},
        "maxDosePerDay": {"class_name": "Quantity", "is_contained": False},
        "maxDosePerTreatmentPeriod": {"class_name": "Ratio", "is_contained": False},
        "maxTreatmentPeriod": {"class_name": "Duration", "is_contained": False},
        "targetSpecies": {"class_name": "TargetSpecies", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "CodeableConcept" = None,
        firstDose: "Quantity" = None,
        maxSingleDose: "Quantity" = None,
        maxDosePerDay: "Quantity" = None,
        maxDosePerTreatmentPeriod: "Ratio" = None,
        maxTreatmentPeriod: "Duration" = None,
        targetSpecies: list["TargetSpecies"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.firstDose = firstDose
        self.maxSingleDose = maxSingleDose
        self.maxDosePerDay = maxDosePerDay
        self.maxDosePerTreatmentPeriod = maxDosePerTreatmentPeriod
        self.maxTreatmentPeriod = maxTreatmentPeriod
        self.targetSpecies = targetSpecies or []

    @classmethod
    def from_dict(cls, data: dict) -> "MedicinalProductPharmaceutical":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "MedicinalProductPharmaceutical":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class MedicinalProductPharmaceutical(DomainResource):
    """A pharmaceutical product described in terms of its composition and dose form.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: An identifier for the pharmaceutical medicinal product
    :param CodeableConcept administrableDoseForm: The administrable dose form, after necessary reconstitution
    :param CodeableConcept unitOfPresentation: Todo
    :param Reference ingredient: Ingredient
    :param Reference device: Accompanying device
    :param Characteristics characteristics: Characteristics e.g. a products onset of action
    :param RouteOfAdministration routeOfAdministration: The path by which the pharmaceutical product is taken into or makes contact with the body
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "administrableDoseForm": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "unitOfPresentation": {"class_name": "CodeableConcept", "is_contained": False},
        "ingredient": {"class_name": "Reference", "is_contained": False},
        "device": {"class_name": "Reference", "is_contained": False},
        "characteristics": {"class_name": "Characteristics", "is_contained": True},
        "routeOfAdministration": {
            "class_name": "RouteOfAdministration",
            "is_contained": True,
        },
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        administrableDoseForm: "CodeableConcept" = None,
        unitOfPresentation: "CodeableConcept" = None,
        ingredient: list["Reference"] = None,
        device: list["Reference"] = None,
        characteristics: list["Characteristics"] = None,
        routeOfAdministration: list["RouteOfAdministration"] = None,
    ):

        self.resourceType = "MedicinalProductPharmaceutical"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.administrableDoseForm = administrableDoseForm
        self.unitOfPresentation = unitOfPresentation
        self.ingredient = ingredient or []
        self.device = device or []
        self.characteristics = characteristics or []
        self.routeOfAdministration = routeOfAdministration or []

    @classmethod
    def from_dict(cls, data: dict) -> "MedicinalProductPharmaceutical":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "MedicinalProductPharmaceutical":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Account. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Period import *
from fhirmodels.R4B.Reference import *
from fhirmodels.R4B.Resource import *


class Coverage(FhirBaseModel):
    """The party(s) that are responsible for covering the payment of this account, and what order should they be applied to the account.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference coverage: The party(s), such as insurances, that may contribute to the payment of this account
    :param int priority: The priority of the coverage in the context of this account
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "coverage": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        coverage: "Reference" = None,
        priority: "int" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.coverage = coverage
        self.priority = priority

    @classmethod
    def from_dict(cls, data: dict) -> "Account":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Account":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Guarantor(FhirBaseModel):
    """The parties responsible for balancing the account if other payment options fall short.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference party: Responsible entity
    :param bool onHold: Credit or other hold applied
    :param Period period: Guarantee account during
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "party": {"class_name": "Reference", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        party: "Reference" = None,
        onHold: "bool" = None,
        period: "Period" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.party = party
        self.onHold = onHold
        self.period = period

    @classmethod
    def from_dict(cls, data: dict) -> "Account":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Account":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Account(DomainResource):
    """A financial tool for tracking value accrued for a particular purpose.  In the healthcare field, used to track charges for a patient, cost centers, etc.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Account number
    :param str status: active | inactive | entered-in-error | on-hold | unknown
    :param CodeableConcept type: E.g. patient, expense, depreciation
    :param str name: Human-readable label
    :param Reference subject: The entity that caused the expenses
    :param Period servicePeriod: Transaction window
    :param Coverage coverage: The party(s) that are responsible for covering the payment of this account, and what order should they be applied to the account
    :param Reference owner: Entity managing the Account
    :param str description: Explanation of purpose/use
    :param Guarantor guarantor: The parties ultimately responsible for balancing the Account
    :param Reference partOf: Reference to a parent Account
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "subject": {"class_name": "Reference", "is_contained": False},
        "servicePeriod": {"class_name": "Period", "is_contained": False},
        "coverage": {"class_name": "Coverage", "is_contained": True},
        "owner": {"class_name": "Reference", "is_contained": False},
        "guarantor": {"class_name": "Guarantor", "is_contained": True},
        "partOf": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        type: "CodeableConcept" = None,
        name: "str" = None,
        subject: list["Reference"] = None,
        servicePeriod: "Period" = None,
        coverage: list["Coverage"] = None,
        owner: "Reference" = None,
        description: "str" = None,
        guarantor: list["Guarantor"] = None,
        partOf: "Reference" = None,
    ):

        self.resourceType = "Account"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.type = type
        self.name = name
        self.subject = subject or []
        self.servicePeriod = servicePeriod
        self.coverage = coverage or []
        self.owner = owner
        self.description = description
        self.guarantor = guarantor or []
        self.partOf = partOf

    @classmethod
    def from_dict(cls, data: dict) -> "Account":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Account":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



import json
import mimetypes
import os
import typing as _t

from fhirmodels import constants as _c


def walk_directory(directory: str) -> _t.Iterator[str]:
    for root, _, filenames in os.walk(directory):
        for filename in filenames:
            yield os.path.join(root, filename)


def read_json_file(file_path: str) -> _t.Any:
    with open(file_path) as f:
        return json.load(f)


def is_json_file(file_path: str) -> bool:
    return mimetypes.guess_type(file_path)[0] == "application/json"


def is_choice_path(path: str):
    return "[x]" in path


def replace_choice_string(path: str, replace: str):
    return path.replace("[x]", replace)


def get_nth_part(path: str, n: int):
    return path.split(".")[n]


def remove_n_parts_from_start(path: str, n: int):
    return ".".join(path.split(".")[n:])


def remove_n_parts_from_end(path: str, n: int):
    return ".".join(path.split(".")[:-n])


def get_path_length(path: str):
    return len(path.split("."))


def is_root_path(path: str):
    if not path:
        return False
    return "." not in path


def join_paths(*paths: str):
    return ".".join(paths)


def get_python_type_for_primitive(fhir_type: str) -> type:
    """Returns the Python type for a FHIR primitive type."""
    if fhir_type in _c.FHIR_PRIMITIVES_TO_PYTHON_MAP:
        return _c.FHIR_PRIMITIVES_TO_PYTHON_MAP[fhir_type]
    elif fhir_type in _c.FHIR_PRIMITIVE_EXPANSION_MAP:
        return _c.FHIR_PRIMITIVES_TO_PYTHON_MAP[
            _c.FHIR_PRIMITIVE_EXPANSION_MAP[fhir_type]
        ]
    else:
        print("Unknown FHIR primitive type: %s", fhir_type)
        return str


def is_primitive_type(fhir_type: str) -> bool:
    """Returns True if the given FHIR type is a primitive type."""
    if fhir_type in _c.FHIR_PRIMITIVES_TO_PYTHON_MAP:
        return True
    elif fhir_type in _c.FHIR_PRIMITIVE_EXPANSION_MAP:
        return True
    else:
        return False



"""
Generated class for AuditEvent. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.Coding import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Quantity import *
from fhirmodels.R5.Range import *
from fhirmodels.R5.Ratio import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *


class Outcome(FhirBaseModel):
    """Indicates whether the event succeeded or failed. A free text descripiton can be given in outcome.text.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Coding code: Whether the event succeeded or failed
    :param CodeableConcept detail: Additional outcome detail
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "code": {"class_name": "Coding", "is_contained": False},
        "detail": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "Coding" = None,
        detail: list["CodeableConcept"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.detail = detail or []

    @classmethod
    def from_dict(cls, data: dict) -> "AuditEvent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "AuditEvent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Agent(FhirBaseModel):
    """An actor taking an active role in the event or activity that is logged.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: How agent participated
    :param CodeableConcept role: Agent role in the event
    :param Reference who: Identifier of who
    :param bool requestor: Whether user is initiator
    :param Reference location: The agent location when the event occurred
    :param str policy: Policy that authorized the agent participation in the event
    :param Reference networkReference: This agent network location for the activity
    :param str networkUri: This agent network location for the activity
    :param str networkString: This agent network location for the activity
    :param CodeableConcept authorization: Allowable authorization for this agent
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "role": {"class_name": "CodeableConcept", "is_contained": False},
        "who": {"class_name": "Reference", "is_contained": False},
        "location": {"class_name": "Reference", "is_contained": False},
        "networkReference": {"class_name": "Reference", "is_contained": False},
        "authorization": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        role: list["CodeableConcept"] = None,
        who: "Reference" = None,
        requestor: "bool" = None,
        location: "Reference" = None,
        policy: list["str"] = None,
        networkReference: "Reference" = None,
        networkUri: "str" = None,
        networkString: "str" = None,
        authorization: list["CodeableConcept"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.role = role or []
        self.who = who
        self.requestor = requestor
        self.location = location
        self.policy = policy or []
        self.networkReference = networkReference
        self.networkUri = networkUri
        self.networkString = networkString
        self.authorization = authorization or []

    @classmethod
    def from_dict(cls, data: dict) -> "AuditEvent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "AuditEvent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Source(FhirBaseModel):
    """The actor that is reporting the event.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference site: Logical source location within the enterprise
    :param Reference observer: The identity of source detecting the event
    :param CodeableConcept type: The type of source where event originated
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "site": {"class_name": "Reference", "is_contained": False},
        "observer": {"class_name": "Reference", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        site: "Reference" = None,
        observer: "Reference" = None,
        type: list["CodeableConcept"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.site = site
        self.observer = observer
        self.type = type or []

    @classmethod
    def from_dict(cls, data: dict) -> "AuditEvent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "AuditEvent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Detail(FhirBaseModel):
    """Tagged value pairs for conveying additional information about the entity.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Name of the property
    :param Quantity valueQuantity: Property value
    :param CodeableConcept valueCodeableConcept: Property value
    :param str valueString: Property value
    :param bool valueBoolean: Property value
    :param int valueInteger: Property value
    :param Range valueRange: Property value
    :param Ratio valueRatio: Property value
    :param str valueTime: Property value
    :param str valueDateTime: Property value
    :param Period valuePeriod: Property value
    :param str valueBase64Binary: Property value
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "valueRange": {"class_name": "Range", "is_contained": False},
        "valueRatio": {"class_name": "Ratio", "is_contained": False},
        "valuePeriod": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        valueQuantity: "Quantity" = None,
        valueCodeableConcept: "CodeableConcept" = None,
        valueString: "str" = None,
        valueBoolean: "bool" = None,
        valueInteger: "int" = None,
        valueRange: "Range" = None,
        valueRatio: "Ratio" = None,
        valueTime: "str" = None,
        valueDateTime: "str" = None,
        valuePeriod: "Period" = None,
        valueBase64Binary: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.valueQuantity = valueQuantity
        self.valueCodeableConcept = valueCodeableConcept
        self.valueString = valueString
        self.valueBoolean = valueBoolean
        self.valueInteger = valueInteger
        self.valueRange = valueRange
        self.valueRatio = valueRatio
        self.valueTime = valueTime
        self.valueDateTime = valueDateTime
        self.valuePeriod = valuePeriod
        self.valueBase64Binary = valueBase64Binary

    @classmethod
    def from_dict(cls, data: dict) -> "AuditEvent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "AuditEvent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Entity(FhirBaseModel):
    """Specific instances of data or objects that have been accessed.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference what: Specific instance of resource
    :param CodeableConcept role: What role the entity played
    :param CodeableConcept securityLabel: Security labels on the entity
    :param str query: Query parameters
    :param Detail detail: Additional Information about the entity
    :param Agent agent: Entity is attributed to this agent
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "what": {"class_name": "Reference", "is_contained": False},
        "role": {"class_name": "CodeableConcept", "is_contained": False},
        "securityLabel": {"class_name": "CodeableConcept", "is_contained": False},
        "detail": {"class_name": "Detail", "is_contained": True},
        "agent": {"class_name": "Agent", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        what: "Reference" = None,
        role: "CodeableConcept" = None,
        securityLabel: list["CodeableConcept"] = None,
        query: "str" = None,
        detail: list["Detail"] = None,
        agent: list["Agent"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.what = what
        self.role = role
        self.securityLabel = securityLabel or []
        self.query = query
        self.detail = detail or []
        self.agent = agent or []

    @classmethod
    def from_dict(cls, data: dict) -> "AuditEvent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "AuditEvent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class AuditEvent(DomainResource):
    """A record of an event relevant for purposes such as operations, privacy, security, maintenance, and performance analysis.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param CodeableConcept category: Type/identifier of event
    :param CodeableConcept code: Specific type of event
    :param str action: Type of action performed during the event
    :param str severity: emergency | alert | critical | error | warning | notice | informational | debug
    :param Period occurredPeriod: When the activity occurred
    :param str occurredDateTime: When the activity occurred
    :param str recorded: Time when the event was recorded
    :param Outcome outcome: Whether the event succeeded or failed
    :param CodeableConcept authorization: Authorization related to the event
    :param Reference basedOn: Workflow authorization within which this event occurred
    :param Reference patient: The patient is the subject of the data used/created/updated/deleted during the activity
    :param Reference encounter: Encounter within which this event occurred or which the event is tightly associated
    :param Agent agent: Actor involved in the event
    :param Source source: Audit Event Reporter
    :param Entity entity: Data or objects used
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "category": {"class_name": "CodeableConcept", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "occurredPeriod": {"class_name": "Period", "is_contained": False},
        "outcome": {"class_name": "Outcome", "is_contained": True},
        "authorization": {"class_name": "CodeableConcept", "is_contained": False},
        "basedOn": {"class_name": "Reference", "is_contained": False},
        "patient": {"class_name": "Reference", "is_contained": False},
        "encounter": {"class_name": "Reference", "is_contained": False},
        "agent": {"class_name": "Agent", "is_contained": True},
        "source": {"class_name": "Source", "is_contained": True},
        "entity": {"class_name": "Entity", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        category: list["CodeableConcept"] = None,
        code: "CodeableConcept" = None,
        action: "str" = None,
        severity: "str" = None,
        occurredPeriod: "Period" = None,
        occurredDateTime: "str" = None,
        recorded: "str" = None,
        outcome: "Outcome" = None,
        authorization: list["CodeableConcept"] = None,
        basedOn: list["Reference"] = None,
        patient: "Reference" = None,
        encounter: "Reference" = None,
        agent: list["Agent"] = None,
        source: "Source" = None,
        entity: list["Entity"] = None,
    ):

        self.resourceType = "AuditEvent"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.category = category or []
        self.code = code
        self.action = action
        self.severity = severity
        self.occurredPeriod = occurredPeriod
        self.occurredDateTime = occurredDateTime
        self.recorded = recorded
        self.outcome = outcome
        self.authorization = authorization or []
        self.basedOn = basedOn or []
        self.patient = patient
        self.encounter = encounter
        self.agent = agent or []
        self.source = source
        self.entity = entity or []

    @classmethod
    def from_dict(cls, data: dict) -> "AuditEvent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "AuditEvent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Consent. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.Attachment import *
from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.Coding import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Expression import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *


class PolicyBasis(FhirBaseModel):
    """A Reference or URL used to uniquely identify the policy the organization will enforce for this Consent. This Reference or URL should be specific to the version of the policy and should be dereferencable to a computable policy of some form.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference reference: Reference backing policy resource
    :param str url: URL to a computable backing policy
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "reference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        reference: "Reference" = None,
        url: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.reference = reference
        self.url = url

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Verification(FhirBaseModel):
    """Whether a treatment instruction (e.g. artificial respiration: yes or no) was verified with the patient, his/her family or another authorized person.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param bool verified: Has been verified
    :param CodeableConcept verificationType: Business case of verification
    :param Reference verifiedBy: Person conducting verification
    :param Reference verifiedWith: Person who verified
    :param str verificationDate: When consent verified
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "verificationType": {"class_name": "CodeableConcept", "is_contained": False},
        "verifiedBy": {"class_name": "Reference", "is_contained": False},
        "verifiedWith": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        verified: "bool" = None,
        verificationType: "CodeableConcept" = None,
        verifiedBy: "Reference" = None,
        verifiedWith: "Reference" = None,
        verificationDate: list["str"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.verified = verified
        self.verificationType = verificationType
        self.verifiedBy = verifiedBy
        self.verifiedWith = verifiedWith
        self.verificationDate = verificationDate or []

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Actor(FhirBaseModel):
    """Who or what is controlled by this provision. Use group to identify a set of actors by some property they share (e.g. 'admitting officers').:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept role: How the actor is involved
    :param Reference reference: Resource for the actor (or group, by role)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "role": {"class_name": "CodeableConcept", "is_contained": False},
        "reference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        role: "CodeableConcept" = None,
        reference: "Reference" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.role = role
        self.reference = reference

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Data(FhirBaseModel):
    """The resources controlled by this provision if specific resources are referenced.:param Period dataPeriod: Timeframe for data controlled by this provision
    :param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str meaning: instance | related | dependents | authoredby
    :param Reference reference: The actual data reference
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "dataPeriod": {"class_name": "Period", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "reference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        dataPeriod: "Period" = None,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        meaning: "str" = None,
        reference: "Reference" = None,
    ):
        self.dataPeriod = dataPeriod
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.meaning = meaning
        self.reference = reference

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Provision(FhirBaseModel):
    """An exception to the base policy of this consent. An exception can be an addition or removal of access permissions.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Period period: Timeframe for this provision
    :param Actor actor: Who|what controlled by this provision (or group, by role)
    :param CodeableConcept action: Actions controlled by this provision
    :param Coding securityLabel: Security Labels that define affected resources
    :param Coding purpose: Context of activities covered by this provision
    :param Coding documentType: e.g. Resource Type, Profile, CDA, etc
    :param Coding resourceType: e.g. Resource Type, Profile, etc
    :param CodeableConcept code: e.g. LOINC or SNOMED CT code, etc. in the content
    :param Period dataPeriod: Timeframe for data controlled by this provision
    :param Data data: Data controlled by this provision
    :param Expression expression: A computable expression of the consent
    :param Provision provision: Nested Exception Provisions
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
        "actor": {"class_name": "Actor", "is_contained": True},
        "action": {"class_name": "CodeableConcept", "is_contained": False},
        "securityLabel": {"class_name": "Coding", "is_contained": False},
        "purpose": {"class_name": "Coding", "is_contained": False},
        "documentType": {"class_name": "Coding", "is_contained": False},
        "resourceType": {"class_name": "Coding", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "dataPeriod": {"class_name": "Period", "is_contained": False},
        "data": {"class_name": "Data", "is_contained": True},
        "expression": {"class_name": "Expression", "is_contained": False},
        "provision": {"class_name": "Provision", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        period: "Period" = None,
        actor: list["Actor"] = None,
        action: list["CodeableConcept"] = None,
        securityLabel: list["Coding"] = None,
        purpose: list["Coding"] = None,
        documentType: list["Coding"] = None,
        resourceType: list["Coding"] = None,
        code: list["CodeableConcept"] = None,
        dataPeriod: "Period" = None,
        data: list["Data"] = None,
        expression: "Expression" = None,
        provision: list["Provision"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.period = period
        self.actor = actor or []
        self.action = action or []
        self.securityLabel = securityLabel or []
        self.purpose = purpose or []
        self.documentType = documentType or []
        self.resourceType = resourceType or []
        self.code = code or []
        self.dataPeriod = dataPeriod
        self.data = data or []
        self.expression = expression
        self.provision = provision or []

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Consent(DomainResource):
    """A record of a healthcare consumer’s  choices  or choices made on their behalf by a third party, which permits or denies identified recipient(s) or recipient role(s) to perform one or more actions within a given policy context, for specific purposes and periods of time.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Identifier for this record (external references)
    :param str status: draft | active | inactive | not-done | entered-in-error | unknown
    :param CodeableConcept category: Classification of the consent statement - for indexing/retrieval
    :param Reference subject: Who the consent applies to
    :param str date: Fully executed date of the consent
    :param Period period: Effective period for this Consent
    :param Reference grantor: Who is granting rights according to the policy and rules
    :param Reference grantee: Who is agreeing to the policy and rules
    :param Reference manager: Consent workflow management
    :param Reference controller: Consent Enforcer
    :param Attachment sourceAttachment: Source from which this consent is taken
    :param Reference sourceReference: Source from which this consent is taken
    :param CodeableConcept regulatoryBasis: Regulations establishing base Consent
    :param PolicyBasis policyBasis: Computable version of the backing policy
    :param Reference policyText: Human Readable Policy
    :param Verification verification: Consent Verified by patient or family
    :param str decision: deny | permit
    :param Provision provision: Constraints to the base Consent.policyRule/Consent.policy
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "category": {"class_name": "CodeableConcept", "is_contained": False},
        "subject": {"class_name": "Reference", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
        "grantor": {"class_name": "Reference", "is_contained": False},
        "grantee": {"class_name": "Reference", "is_contained": False},
        "manager": {"class_name": "Reference", "is_contained": False},
        "controller": {"class_name": "Reference", "is_contained": False},
        "sourceAttachment": {"class_name": "Attachment", "is_contained": False},
        "sourceReference": {"class_name": "Reference", "is_contained": False},
        "regulatoryBasis": {"class_name": "CodeableConcept", "is_contained": False},
        "policyBasis": {"class_name": "PolicyBasis", "is_contained": True},
        "policyText": {"class_name": "Reference", "is_contained": False},
        "verification": {"class_name": "Verification", "is_contained": True},
        "provision": {"class_name": "Provision", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        category: list["CodeableConcept"] = None,
        subject: "Reference" = None,
        date: "str" = None,
        period: "Period" = None,
        grantor: list["Reference"] = None,
        grantee: list["Reference"] = None,
        manager: list["Reference"] = None,
        controller: list["Reference"] = None,
        sourceAttachment: list["Attachment"] = None,
        sourceReference: list["Reference"] = None,
        regulatoryBasis: list["CodeableConcept"] = None,
        policyBasis: "PolicyBasis" = None,
        policyText: list["Reference"] = None,
        verification: list["Verification"] = None,
        decision: "str" = None,
        provision: list["Provision"] = None,
    ):

        self.resourceType = "Consent"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.category = category or []
        self.subject = subject
        self.date = date
        self.period = period
        self.grantor = grantor or []
        self.grantee = grantee or []
        self.manager = manager or []
        self.controller = controller or []
        self.sourceAttachment = sourceAttachment or []
        self.sourceReference = sourceReference or []
        self.regulatoryBasis = regulatoryBasis or []
        self.policyBasis = policyBasis
        self.policyText = policyText or []
        self.verification = verification or []
        self.decision = decision
        self.provision = provision or []

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Signature. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.generator import FhirBaseModel
from fhirmodels.R4B.Coding import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Reference import *


class Signature(FhirBaseModel):
    """Base StructureDefinition for Signature Type: A signature along with supporting context. The signature may be a digital signature that is cryptographic in nature, or some other signature acceptable to the domain. This other signature may be as simple as a graphical image representing a hand-written signature, or a signature ceremony Different signature approaches have different utilities.
    :param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Coding type: Indication of the reason the entity signed the object(s)
    :param str when: When the signature was created
    :param Reference who: Who signed
    :param Reference onBehalfOf: The party represented
    :param str targetFormat: The technical format of the signed resources
    :param str sigFormat: The technical format of the signature
    :param str data: The actual signature content (XML DigSig. JWS, picture, etc.)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "Coding", "is_contained": False},
        "who": {"class_name": "Reference", "is_contained": False},
        "onBehalfOf": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        type: list["Coding"] = None,
        when: "str" = None,
        who: "Reference" = None,
        onBehalfOf: "Reference" = None,
        targetFormat: "str" = None,
        sigFormat: "str" = None,
        data: "str" = None,
    ):

        self.id = id
        self.extension = extension or []
        self.type = type or []
        self.when = when
        self.who = who
        self.onBehalfOf = onBehalfOf
        self.targetFormat = targetFormat
        self.sigFormat = sigFormat
        self.data = data

    @classmethod
    def from_dict(cls, data: dict) -> "Signature":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Signature":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for BiologicallyDerivedProduct. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.Attachment import *
from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.Coding import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Quantity import *
from fhirmodels.R5.Range import *
from fhirmodels.R5.Ratio import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *


class Collection(FhirBaseModel):
    """How this product was collected.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference collector: Individual performing collection
    :param Reference source: The patient who underwent the medical procedure to collect the product or the organization that facilitated the collection
    :param str collectedDateTime: Time of product collection
    :param Period collectedPeriod: Time of product collection
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "collector": {"class_name": "Reference", "is_contained": False},
        "source": {"class_name": "Reference", "is_contained": False},
        "collectedPeriod": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        collector: "Reference" = None,
        source: "Reference" = None,
        collectedDateTime: "str" = None,
        collectedPeriod: "Period" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.collector = collector
        self.source = source
        self.collectedDateTime = collectedDateTime
        self.collectedPeriod = collectedPeriod

    @classmethod
    def from_dict(cls, data: dict) -> "BiologicallyDerivedProduct":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "BiologicallyDerivedProduct":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Property(FhirBaseModel):
    """A property that is specific to this BiologicallyDerviedProduct instance.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Code that specifies the property
    :param bool valueBoolean: Property values
    :param int valueInteger: Property values
    :param CodeableConcept valueCodeableConcept: Property values
    :param Period valuePeriod: Property values
    :param Quantity valueQuantity: Property values
    :param Range valueRange: Property values
    :param Ratio valueRatio: Property values
    :param str valueString: Property values
    :param Attachment valueAttachment: Property values
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "valueCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "valuePeriod": {"class_name": "Period", "is_contained": False},
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueRange": {"class_name": "Range", "is_contained": False},
        "valueRatio": {"class_name": "Ratio", "is_contained": False},
        "valueAttachment": {"class_name": "Attachment", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        valueBoolean: "bool" = None,
        valueInteger: "int" = None,
        valueCodeableConcept: "CodeableConcept" = None,
        valuePeriod: "Period" = None,
        valueQuantity: "Quantity" = None,
        valueRange: "Range" = None,
        valueRatio: "Ratio" = None,
        valueString: "str" = None,
        valueAttachment: "Attachment" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.valueBoolean = valueBoolean
        self.valueInteger = valueInteger
        self.valueCodeableConcept = valueCodeableConcept
        self.valuePeriod = valuePeriod
        self.valueQuantity = valueQuantity
        self.valueRange = valueRange
        self.valueRatio = valueRatio
        self.valueString = valueString
        self.valueAttachment = valueAttachment

    @classmethod
    def from_dict(cls, data: dict) -> "BiologicallyDerivedProduct":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "BiologicallyDerivedProduct":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class BiologicallyDerivedProduct(DomainResource):
    """A biological material originating from a biological entity intended to be transplanted or infused into another (possibly the same) biological entity.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Coding productCategory: organ | tissue | fluid | cells | biologicalAgent
    :param CodeableConcept productCode: A code that identifies the kind of this biologically derived product
    :param Reference parent: The parent biologically-derived product
    :param Reference request: Request to obtain and/or infuse this product
    :param Identifier identifier: Instance identifier
    :param Identifier biologicalSourceEvent: An identifier that supports traceability to the event during which material in this product from one or more biological entities was obtained or pooled
    :param Reference processingFacility: Processing facilities responsible for the labeling and distribution of this biologically derived product
    :param str division: A unique identifier for an aliquot of a product
    :param Coding productStatus: available | unavailable
    :param str expirationDate: Date, and where relevant time, of expiration
    :param Collection collection: How this product was collected
    :param Range storageTempRequirements: Product storage temperature requirements
    :param Property property: A property that is specific to this BiologicallyDerviedProduct instance
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "productCategory": {"class_name": "Coding", "is_contained": False},
        "productCode": {"class_name": "CodeableConcept", "is_contained": False},
        "parent": {"class_name": "Reference", "is_contained": False},
        "request": {"class_name": "Reference", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "biologicalSourceEvent": {"class_name": "Identifier", "is_contained": False},
        "processingFacility": {"class_name": "Reference", "is_contained": False},
        "productStatus": {"class_name": "Coding", "is_contained": False},
        "collection": {"class_name": "Collection", "is_contained": True},
        "storageTempRequirements": {"class_name": "Range", "is_contained": False},
        "property": {"class_name": "Property", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        productCategory: "Coding" = None,
        productCode: "CodeableConcept" = None,
        parent: list["Reference"] = None,
        request: list["Reference"] = None,
        identifier: list["Identifier"] = None,
        biologicalSourceEvent: "Identifier" = None,
        processingFacility: list["Reference"] = None,
        division: "str" = None,
        productStatus: "Coding" = None,
        expirationDate: "str" = None,
        collection: "Collection" = None,
        storageTempRequirements: "Range" = None,
        property: list["Property"] = None,
    ):

        self.resourceType = "BiologicallyDerivedProduct"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.productCategory = productCategory
        self.productCode = productCode
        self.parent = parent or []
        self.request = request or []
        self.identifier = identifier or []
        self.biologicalSourceEvent = biologicalSourceEvent
        self.processingFacility = processingFacility or []
        self.division = division
        self.productStatus = productStatus
        self.expirationDate = expirationDate
        self.collection = collection
        self.storageTempRequirements = storageTempRequirements
        self.property = property or []

    @classmethod
    def from_dict(cls, data: dict) -> "BiologicallyDerivedProduct":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "BiologicallyDerivedProduct":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()
